# -*- coding: utf-8 -*-
"""Testlink Managers"""


from qatestlink.core.connections.connection_base import ConnectionBase
from qatestlink.core.logger_manager import LoggerManager
from qatestlink.core.models.tl_models import (
    TBuild, TCase, TPlan, TPlatform, TProject, TSuite
)
from qatestlink.core.models.tl_reports import (
    RTCase, RTPlanTotals
)
from qatestlink.core.utils import settings as settings_func
from qatestlink.core.xmls.xmlrpc_manager import XMLRPCManager


PATH_CONFIG = 'qatestlink/configs/settings.json'


class TLManager(object):
    """
    This class allows to send, handle and interpretate reponses
     to/from testlink api XMLRPC
    """

    _settings_path = None
    _settings = None
    _logger_manager = None
    _xml_manager = None
    _conn = None

    log = None

    def __init__(self, file_path=None, file_name=None, settings=None):
        """
        This instance allow to handle requests+reponses to/from XMLRPC
        just need settings_path and settings dict loaded to works

        :Args:
            settings_path: Path for search JSON
                file with settings values
            settings: Load settings at default path,
                'qatestlink/configs/settings.json'
        """
        if not settings:
            if not file_path or not file_name:
                settings = settings_func()
            else:
                settings = settings_func(
                    file_path=file_path,
                    file_name=file_name)
        self._settings = settings
        self._logger_manager = LoggerManager(
            self._settings.get('log_level'))
        self.log = self._logger_manager.log
        self._xml_manager = XMLRPCManager(self.log)
        # generate url using settings
        conn = self._settings.get('connection')
        self._conn = ConnectionBase(
            self.log,
            host=conn.get('host'),
            port=conn.get('port'),
            is_https=conn.get('is_https'))

    def api_login(self, dev_key=None):
        """Call to method named 'checkDevKey' for testlink XMLRPC

        Keyword Arguments:
            dev_key {str} -- string of developer key provided by Testlink
                (default: {value obtained from JSON settings file})

        Returns:
            bool -- check if developer key it's valid for Testlink
        """
        if not dev_key:
            dev_key = self._settings.get('dev_key')
        req_data = self._xml_manager.req_check_dev_key(dev_key)
        res = self._conn.post(self._xml_manager.headers, req_data)
        res_dict = self._xml_manager.parse_response(res)
        res_value = res_dict.get(
            'methodResponse')['params']['param']['value']
        if res_value.get('boolean'):
            return bool(res_value.get('boolean'))
        self._xml_manager.parse_errors(res_dict)

    def api_tprojects(self, dev_key=None):
        """Call to method named 'tl.getProjects' for testlink XMLRPC

        Keyword Arguments:
            dev_key {str} -- string of developer key provided by Testlink
                (default: {value obtained from JSON settings file})

        Returns:
            list(TProject) -- list of object model for Testlink Test Project
                data
        """
        if not dev_key:
            dev_key = self._settings.get('dev_key')
        req_data = self._xml_manager.req_tprojects(dev_key)
        res = self._conn.post(self._xml_manager.headers, req_data)
        res_dict = self._xml_manager.parse_response(res)
        res_param = res_dict.get(
            'methodResponse')['params']['param']['value']
        data_list = res_param.get('array')['data']['value']
        tprojects = list()
        for data_properties in data_list:
            properties = data_properties['struct']['member']
            tproject = TProject(properties)
            tprojects.append(tproject)
        return tprojects

    def api_tproject(self, tproject_name, dev_key=None):
        """Call to method named 'tl.getTestProjectByName' for testlink XMLRPC

        Arguments:
            tproject_name {str} -- name of testlink Test Project

        Keyword Arguments:
            dev_key {str} -- string of developer key provided by Testlink
                (default: {value obtained from JSON settings file})

        Returns:
            TProject -- object model for Testlink Test Project data
        """
        if not dev_key:
            dev_key = self._settings.get('dev_key')
        req_data = self._xml_manager.req_tproject_by_name(
            dev_key, tproject_name)
        res = self._conn.post(self._xml_manager.headers, req_data)
        res_dict = self._xml_manager.parse_response(res)
        res_param = res_dict.get(
            'methodResponse')['params']['param']['value']
        data_properties = res_param['struct']['member']
        return TProject(data_properties)

    def api_tproject_tplans(self, tproject_id, dev_key=None):
        """Call to method named 'tl.getProjectTestPlans' for testlink XMLRPC

        Arguments:
            tproject_id {int} -- ID of Testlink Test Project to get Testlink
                Test Plan

        Keyword Arguments:
            dev_key {str} -- string of developer key provided by Testlink
                (default: {value obtained from JSON settings file})

        Returns:
            list(TPlan) -- list of object model for Testlink Test Plan data
        """
        if not dev_key:
            dev_key = self._settings.get('dev_key')
        req_data = self._xml_manager.req_tproject_tplans(
            dev_key, tproject_id)
        res = self._conn.post(self._xml_manager.headers, req_data)
        res_dict = self._xml_manager.parse_response(res)
        res_param = res_dict.get(
            'methodResponse')['params']['param']['value']
        data_list = res_param.get('array')['data']['value']
        tplans = list()
        for data_properties in data_list:
            properties = data_properties['struct']['member']
            tplan = TPlan(properties)
            tplans.append(tplan)
        return tplans

    def api_tproject_tsuites_first_level(self, tproject_id, dev_key=None):
        """Call to method named 'tl.getFirstLevelTestSuitesForTestProject' for
            testlink XMLRPC

        Arguments:
            tproject_id {int} -- ID of Testlink Test Project to get Testlink
                Test Plan

        Keyword Arguments:
            dev_key {str} -- string of developer key provided by Testlink
                (default: {value obtained from JSON settings file})

        Returns:
            list(TSuite) -- list of object model for Testlink Test Suite
                data
        """
        if not dev_key:
            dev_key = self._settings.get('dev_key')
        req_data = self._xml_manager.req_tproject_tsuites_first_level(
            dev_key, tproject_id)
        res = self._conn.post(self._xml_manager.headers, req_data)
        res_dict = self._xml_manager.parse_response(res)
        res_param = res_dict.get(
            'methodResponse')['params']['param']['value']
        data_list = res_param.get('array')['data']['value']
        tsuites = list()
        try:
            for data_properties in data_list:
                properties = data_properties['struct']['member']
                tsuite = TSuite(properties)
                tsuites.append(tsuite)
            return tsuites
        except TypeError:
            raise self._xml_manager.parse_errors(res_dict)

    def api_tplan(self, tproject_name, tplan_name, dev_key=None):
        """Call to method named 'tl.getTestPlanByName' for testlink XMLRPC

        Arguments:
            tproject_name {str} -- NAME of Testlink Test Project to get
                Testlink Test Project
            tplan_name {str} -- NAME of Testlink Test Project to get Testlink
                Test Plan

        Keyword Arguments:
            dev_key {str} -- string of developer key provided by Testlink
                (default: {value obtained from JSON settings file})

        Returns:
            TPlan -- object model for Testlink Test Plan data
        """
        if not dev_key:
            dev_key = self._settings.get('dev_key')
        req_data = self._xml_manager.req_tplan_by_name(
            dev_key, tproject_name, tplan_name)
        res = self._conn.post(self._xml_manager.headers, req_data)
        res_dict = self._xml_manager.parse_response(res)
        res_param = res_dict.get(
            'methodResponse')['params']['param']['value']
        data_properties = res_param.get(
            'array')['data']['value']['struct']['member']
        return TPlan(data_properties)

    def api_tplan_platforms(self, tplan_id, dev_key=None):
        """Call to method named 'tl.getTestPlanPlatforms' for testlink XMLRPC

        Arguments:
            tplan_id {int} -- ID of Testlink Test Project to get Testlink
                Test Plan

        Keyword Arguments:
            dev_key {str} -- string of developer key provided by Testlink
                (default: {value obtained from JSON settings file})

        Returns:
            list(TPlatform) -- list of object model for Testlink Platform data
        """
        if dev_key is None:
            dev_key = self._settings.get('dev_key')
        req_data = self._xml_manager.req_tplan_platforms(dev_key, tplan_id)
        res = self._conn.post(self._xml_manager.headers, req_data)
        res_dict = self._xml_manager.parse_response(res)
        res_param = res_dict.get(
            'methodResponse')['params']['param']['value']
        data_list = res_param.get('array')['data']['value']
        tplatforms = list()
        for data_properties in data_list:
            properties = data_properties['struct']['member']
            tplatform = TPlatform(properties)
            tplatforms.append(tplatform)
        return tplatforms

    def api_tplan_builds(self, tplan_id, dev_key=None):
        """Call to method named 'tl.getBuildsForTestPlan' for testlink XMLRPC

        Arguments:
            tplan_id {int} -- ID of Testlink Test Project to get Testlink
                Test Plan

        Keyword Arguments:
            dev_key {str} -- string of developer key provided by Testlink
                (default: {value obtained from JSON settings file})

        Returns:
            list(TBuild) -- list of object model for Testlink Build data
        """
        if not dev_key:
            dev_key = self._settings.get('dev_key')
        req_data = self._xml_manager.req_tplan_builds(dev_key, tplan_id)
        res = self._conn.post(self._xml_manager.headers, req_data)
        res_dict = self._xml_manager.parse_response(res)
        res_param = res_dict.get(
            'methodResponse')['params']['param']['value']
        data_list = res_param.get('array')['data']['value']
        tbuilds = list()
        for data_properties in data_list:
            properties = data_properties['struct']['member']
            tbuild = TBuild(properties)
            tbuilds.append(tbuild)
        return tbuilds

    def api_tplan_tsuites(self, tplan_id, dev_key=None):
        """Call to method named 'tl.getTestSuitesForTestPlan' for testlink
            XMLRPC

        Arguments:
            tplan_id {int} -- ID of Testlink Test Project to get Testlink
                Test Plan

        Keyword Arguments:
            dev_key {str} -- string of developer key provided by Testlink
                (default: {value obtained from JSON settings file})

        Returns:
            list(TSuite) -- list of object model for Testlink Test Suite data
        """
        if not dev_key:
            dev_key = self._settings.get('dev_key')
        req_data = self._xml_manager.req_tplan_tsuites(dev_key, tplan_id)
        res = self._conn.post(self._xml_manager.headers, req_data)
        res_dict = self._xml_manager.parse_response(res)
        res_param = res_dict.get(
            'methodResponse')['params']['param']['value']
        data_list = res_param.get('array')['data']['value']
        tsuites = list()
        try:
            for data_properties in data_list:
                properties = data_properties['struct']['member']
                tsuite = TSuite(properties)
                tsuites.append(tsuite)
            return tsuites
        except TypeError:
            raise self._xml_manager.parse_errors(res_dict)

    def api_tplan_tcases(self, tplan_id, dev_key=None):
        """Call to method named 'tl.getTestCasesForTestPlan' for testlink
            XMLRPC

        Arguments:
            tplan_id {int} -- ID of Testlink Test Project to get Testlink
                Test Plan

        Keyword Arguments:
            dev_key {str} -- string of developer key provided by Testlink
                (default: {value obtained from JSON settings file})

        Returns:
            list(TCase) -- list of object model for Testlink Test Case data
        """
        if dev_key is None:
            dev_key = self._settings.get('dev_key')
        req_data = self._xml_manager.req_tplan_tcases(dev_key, tplan_id)
        res = self._conn.post(self._xml_manager.headers, req_data)
        res_dict = self._xml_manager.parse_response(res)
        res_param = res_dict.get(
            'methodResponse')['params']['param']['value']
        data_list = res_param.get('struct')['member']
        tcases = list()
        for data_properties in data_list:
            # TODO: make all assigned builds reporting to models,
            # not just first
            properties = data_properties.get(
                'value')['struct']['member'][0]['value']['struct']['member']
            tcase = TCase(properties)
            tcases.append(tcase)
        return tcases

    def api_tplan_build_latest(self, tplan_id, dev_key=None):
        """TODO: doc"""
        if dev_key is None:
            dev_key = self._settings.get('dev_key')
        req_data = self._xml_manager.req_tplan_build_latest(dev_key, tplan_id)
        res = self._conn.post(self._xml_manager.headers, req_data)
        res_dict = self._xml_manager.parse_response(res)
        res_param = res_dict.get(
            'methodResponse')['params']['param']['value']
        properties = res_param.get('struct')['member']
        return TBuild(properties)

    def api_tplan_totals(self, tplan_id, dev_key=None):
        """TODO: doc"""
        if dev_key is None:
            dev_key = self._settings.get('dev_key')
        req_data = self._xml_manager.req_tplan_totals(dev_key, tplan_id)
        res = self._conn.post(self._xml_manager.headers, req_data)
        res_dict = self._xml_manager.parse_response(res)
        res_param = res_dict.get(
            'methodResponse')['params']['param']['value']
        properties = res_param.get('struct')['member']
        return RTPlanTotals(properties)

    def api_tsuite(self, tsuite_id, dev_key=None):
        """TODO: doc"""
        if dev_key is None:
            dev_key = self._settings.get('dev_key')
        req_data = self._xml_manager.req_tsuite_by_id(dev_key, tsuite_id)
        res = self._conn.post(self._xml_manager.headers, req_data)
        res_dict = self._xml_manager.parse_response(res)
        try:
            res_param = res_dict.get(
                'methodResponse')['params']['param']['value']
            properties = res_param.get('struct')['member']
            return TSuite(properties)
        except TypeError:
            raise self._xml_manager.parse_errors(res_dict)

    def api_tsuite_tsuites(self, tsuite_id, dev_key=None):
        """TODO: doc"""
        if dev_key is None:
            dev_key = self._settings.get('dev_key')
        req_data = self._xml_manager.req_tsuite_tsuites_by_id(
            dev_key, tsuite_id)
        res = self._conn.post(self._xml_manager.headers, req_data)
        res_dict = self._xml_manager.parse_response(res)
        res_param = res_dict.get(
            'methodResponse')['params']['param']['value']
        data_list = res_param.get('struct')['member']
        tsuites = list()
        for data_properties in data_list:
            properties = data_properties['value']['struct']['member']
            tsuite = TSuite(properties)
            tsuites.append(tsuite)
        return tsuites

    def api_tcase(self, tcase_id=None, external_id=None, dev_key=None):
        """TODO: doc"""
        if dev_key is None:
            dev_key = self._settings.get('dev_key')
        req_data = self._xml_manager.req_tcase_by_id_or_external(
            dev_key, tcase_id=tcase_id, external_id=external_id)
        res = self._conn.post(self._xml_manager.headers, req_data)
        res_dict = self._xml_manager.parse_response(res)
        res_param = res_dict.get(
            'methodResponse')['params']['param']['value']
        properties = res_param.get(
            'array')['data']['value']['struct']['member']
        return TCase(properties)

    def api_tcase_by_name(self, tcase_name, dev_key=None):
        """TODO: doc"""
        if dev_key is None:
            dev_key = self._settings.get('dev_key')
        req_data = self._xml_manager.req_tcase_by_name(
            dev_key, tcase_name)
        res = self._conn.post(self._xml_manager.headers, req_data)
        res_dict = self._xml_manager.parse_response(res)
        res_param = res_dict.get(
            'methodResponse')['params']['param']['value']
        properties = res_param.get(
            'array')['data']['value']['struct']['member']
        return TCase(properties)

    def api_tcase_report(self, **kwargs):
        """Reports a result for a single test case

        Keyword Arguments:
            tcase_id {[type]} -- [description] (default: {None})
            external_id {[type]} -- [description] (default: {None})
            tplan_id {[type]} -- [description] (default: {None})
            status {[type]} -- [description] (default: {None})
            build_id {[type]} -- [description] (default: {None})
            build_name {[type]} -- [description] (default: {None})
            notes {[type]} -- [description] (default: {None})
            duration {[type]} -- [description] (default: {None})
            guess {[type]} -- [description] (default: {None})
            bug_id {[type]} -- [description] (default: {None})
            platform_id {[type]} -- [description] (default: {None})
            platform_name {[type]} -- [description] (default: {None})
            custom_fields {[type]} -- [description] (default: {None})
            overwrite {[type]} -- [description] (default: {None})
            user_name {[type]} -- [description] (default: {None})
            timestamp {[type]} -- [description] (default: {None})
            dev_key {[type]} -- [description] (default: {None})

        Returns:
            [type] -- [description]
        """
        if not kwargs.get('dev_key'):
            kwargs['dev_key'] = self._settings.get('dev_key')
        req_data = self._xml_manager.req_tcase_report(**kwargs)
        res = self._conn.post(self._xml_manager.headers, req_data)
        res_dict = self._xml_manager.parse_response(res)
        res_param = res_dict.get(
            'methodResponse')['params']['param']['value']
        properties = res_param.get(
            'array')['data']['value']['struct']['member']
        return RTCase(properties)

    def api_user_exist(self, user_name, dev_key=None):
        """Call to method named 'tl.doesUserExist' for testlink XMLRPC

        Keyword Arguments:
            user_name {str} -- NAME of remote testlink user
            dev_key {str} -- string of developer key provided by Testlink
                (default: {value obtained from JSON settings file})

        Returns:
            bool -- check if user name it's valid for Testlink
        """
        if not dev_key:
            dev_key = self._settings.get('dev_key')
        req_data = self._xml_manager.req_user_exist(dev_key, user_name)
        res = self._conn.post(self._xml_manager.headers, req_data)
        res_dict = self._xml_manager.parse_response(res)
        res_value = res_dict.get(
            'methodResponse')['params']['param']['value']
        if res_value.get('boolean'):
            return bool(res_value.get('boolean'))
        self._xml_manager.parse_errors(res_dict)

    def api_about(self):
        """Call to method named 'tl.about' for testlink XMLRPC

        Returns:
            str -- Message predefined by Testlink code
        """
        req_data = self._xml_manager.req_about()
        res = self._conn.post(self._xml_manager.headers, req_data)
        res_dict = self._xml_manager.parse_response(res)
        res_value = res_dict.get(
            'methodResponse')['params']['param']['value']
        return str(res_value.get('string'))

    def api_say_hello(self):
        """Call to method named 'tl.sayHello' for testlink XMLRPC

        Returns:
            str -- Message predefined by Testlink code
        """
        req_data = self._xml_manager.req_say_hello()
        res = self._conn.post(self._xml_manager.headers, req_data)
        res_dict = self._xml_manager.parse_response(res)
        res_value = res_dict.get(
            'methodResponse')['params']['param']['value']
        return str(res_value.get('string'))

    def api_ping(self):
        """Call to method named 'tl.ping' for testlink XMLRPC

        Returns:
            str -- Message predefined by Testlink code
        """
        req_data = self._xml_manager.req_ping()
        res = self._conn.post(self._xml_manager.headers, req_data)
        res_dict = self._xml_manager.parse_response(res)
        res_value = res_dict.get(
            'methodResponse')['params']['param']['value']
        return str(res_value.get('string'))

    def api_repeat(self, repeat):
        """Call to method named 'tl.repeat' for testlink XMLRPC

        Returns:
            str -- Message predefined by Testlink code
        """
        req_data = self._xml_manager.req_repeat(repeat)
        res = self._conn.post(self._xml_manager.headers, req_data)
        res_dict = self._xml_manager.parse_response(res)
        res_value = res_dict.get(
            'methodResponse')['params']['param']['value']
        return str(res_value.get('string'))



# -*- coding: utf-8 -*-
"""TODO: doc module"""


class ResponseException(Exception):
    """Inherits exception, just for raises testlink XMLRPC errors"""

    def __init__(self, log, code=-1,
                 message='Response exception message not defined',
                 err=None):
        """Raise an exception from any part of qacode package"""
        super(ResponseException, self).__init__(err, message)
        self._code = code
        self._message = message
        log.error(("Response exception detected: \n    code={}\n"
                   "    message={}").format(
                       self._code, self._message))



# -*- coding: utf-8 -*-
# pylint: disable=invalid-name
"""TODO: doc module"""


import logging
from unittest import TestCase
from unittest import skipIf
from qatestlink.core.exceptions.response_exception import ResponseException
from qatestlink.core.models.tl_models import (
    TBuild, TCase, TPlan, TPlatform, TProject, TSuite
)
from qatestlink.core.models.tl_reports import (
    RTCase, RTPlanTotals
)
from qatestlink.core.testlink_manager import TLManager
from qatestlink.core.utils import settings


SETTINGS = settings()
API_DEV_KEY = SETTINGS['dev_key']
SKIP = SETTINGS['tests']['skip']['methods']
SKIP_MESSAGE = SETTINGS['tests']['skip_message']
DATA = SETTINGS['tests']['data']


class TestMethods(TestCase):
    """TODO: doc class"""

    @classmethod
    def setUpClass(cls):
        """TODO: doc method"""
        cls.testlink_manager = TLManager()

    def setUp(self):
        """TODO: doc method"""
        self.assertIsInstance(
            self.testlink_manager, TLManager)
        self.assertIsInstance(
            self.testlink_manager.log, logging.Logger)

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_001_method_tprojects(self):
        """TODO: doc method"""
        tprojects = self.testlink_manager.api_tprojects(
            dev_key=API_DEV_KEY)
        self.assertIsInstance(tprojects, list)
        self.assertGreater(len(tprojects), 0)
        for tproject in tprojects:
            self.testlink_manager.log.debug(repr(tproject))
            self.assertIsInstance(tproject, TProject)

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_002_method_tproject(self):
        """TODO: doc method"""
        tproject = self.testlink_manager.api_tproject(DATA['tproject_name'])
        self.assertIsInstance(tproject, TProject)
        self.assertEquals(tproject.name, DATA['tproject_name'])

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_003_method_tproject_tplans(self):
        """TODO: doc method"""
        tplans = self.testlink_manager.api_tproject_tplans(DATA['tproject_id'])
        self.assertIsInstance(tplans, list)
        self.assertGreater(len(tplans), 0)
        for tplan in tplans:
            self.testlink_manager.log.debug(repr(tplan))
            self.assertIsInstance(tplan, TPlan)

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_004_method_tproject_tsuites_first_level(self):
        """TODO: doc method"""
        tsuites = self.testlink_manager.api_tproject_tsuites_first_level(
            DATA['tproject_id'])
        self.assertIsInstance(tsuites, list)
        self.assertGreater(len(tsuites), 0)
        for tsuite in tsuites:
            self.testlink_manager.log.debug(repr(tsuite))
            self.assertIsInstance(tsuite, TSuite)

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_005_method_tplan(self):
        """TODO: doc method"""
        tplan = self.testlink_manager.api_tplan(
            DATA['tproject_name'], DATA['tplan_name'])
        self.assertIsInstance(tplan, TPlan)
        self.assertEquals(tplan.name, DATA['tplan_name'])
        self.assertEquals(tplan.tproject_id, DATA['tproject_id'])

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_006_method_tplan_platforms(self):
        """TODO: doc method"""
        platforms = self.testlink_manager.api_tplan_platforms(
            DATA['tplan_id'])
        self.assertIsInstance(platforms, list)
        self.assertGreater(len(platforms), 0)
        for platform in platforms:
            self.testlink_manager.log.debug(repr(platform))
            self.assertIsInstance(platform, TPlatform)

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_007_method_tplan_builds(self):
        """TODO: doc method"""
        builds = self.testlink_manager.api_tplan_builds(DATA['tplan_id'])
        self.assertIsInstance(builds, list)
        self.assertGreater(len(builds), 0)
        for build in builds:
            self.testlink_manager.log.debug(repr(build))
            self.assertIsInstance(build, TBuild)

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_008_method_tplan_tsuites(self):
        """TODO: doc method"""
        tsuites = self.testlink_manager.api_tplan_tsuites(DATA['tplan_id'])
        self.assertIsInstance(tsuites, list)
        self.assertGreater(len(tsuites), 0)
        for tsuite in tsuites:
            self.testlink_manager.log.debug(repr(tsuite))
            self.assertIsInstance(tsuite, TSuite)

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_009_method_tplan_tcases(self):
        """TODO: doc method"""
        tcases = self.testlink_manager.api_tplan_tcases(DATA['tplan_id'])
        self.assertIsInstance(tcases, list)
        self.assertGreater(len(tcases), 0)
        for tcase in tcases:
            self.testlink_manager.log.debug(repr(tcase))
            self.assertIsInstance(tcase, TCase)
            if tcase.id == DATA['tcase_id']:
                self.assertEquals(
                    tcase.full_external_id,
                    DATA['tcase_full_external_id']
                )

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_010_method_tplan_tbuild_latest(self):
        """TODO: doc method"""
        build = self.testlink_manager.api_tplan_build_latest(
            DATA['tplan_id'])
        self.assertIsInstance(build, TBuild)
        self.assertEquals(build.id, DATA['build_id_two'])
        self.assertEquals(build.name, DATA['build_name_two'])

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_011_method_tplan_totals(self):
        """TODO: doc method"""
        totals = self.testlink_manager.api_tplan_totals(
            DATA['tplan_id'])
        self.assertIsInstance(totals, RTPlanTotals)
        self.assertIsInstance(totals.by_tester, list)
        self.assertIsInstance(totals.by_tester[0]['user_id'], int)
        self.assertIsInstance(totals.by_tester[0]['report_types'], list)
        for by_tester_report_type in totals.by_tester[0]['report_types']:
            self.assertIsInstance(by_tester_report_type['platform_id'], int)
            self.assertIsInstance(by_tester_report_type['qty'], int)
            self.assertIsInstance(by_tester_report_type['report_type'], str)
            self.assertIn(
                by_tester_report_type['report_type'],
                ['p', 'n', 'b', 'f']
            )

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_012_method_tsuite(self):
        """TODO: doc method"""
        tsuite = self.testlink_manager.api_tsuite(
            DATA['tsuite_id'])
        self.assertIsInstance(tsuite, TSuite)
        self.assertEquals(tsuite.name, DATA['tsuite_name'])

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_013_method_tsuite_tsuites(self):
        """TODO: doc method"""
        tsuites = self.testlink_manager.api_tsuite_tsuites(
            DATA['tsuite_id'])
        self.assertIsInstance(tsuites, list)
        for tsuite in tsuites:
            self.assertIsInstance(tsuite, TSuite)

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_014_method_tcase_byid(self):
        """TODO: doc method"""
        tcase = self.testlink_manager.api_tcase(
            tcase_id=DATA['tcase_id'])
        self.assertIsInstance(tcase, TCase)
        self.assertEquals(tcase.id, DATA['tcase_id'])

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_015_method_tcase_byexternalid(self):
        """TODO: doc method"""
        tcase = self.testlink_manager.api_tcase(
            external_id=DATA['tcase_full_external_id'])
        self.assertIsInstance(tcase, TCase)
        self.assertEquals(
            tcase.external_id, DATA['tcase_full_external_id'])

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_016_method_tcase_byname(self):
        """TODO: doc method"""
        tcase = self.testlink_manager.api_tcase_by_name(
            DATA['tcase_name'])
        self.assertIsInstance(tcase, TCase)
        self.assertEquals(
            tcase.name, DATA['tcase_name'])

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_017_method_tcase_report(self):
        """TODO: doc method"""
        report = self.testlink_manager.api_tcase_report(
            external_id=DATA['tc_report']['external_id'],
            tplan_id=DATA['tc_report']['tplan_id'],
            build_id=DATA['tc_report']['build_id'],
            platform_id=DATA['tc_report']['platform_id'],
            status=DATA['tc_report']['status']['blocked']
        )
        self.assertIsInstance(report, RTCase)
        self.assertTrue(report.status)
        self.assertEquals(
            report.message, DATA['tc_report']['message'])

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_018_method_user_exist(self):
        """TODO: doc method"""
        is_user = self.testlink_manager.api_user_exist(
            DATA['user_name'])
        self.assertTrue(is_user)

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_019_method_about(self):
        """TODO: doc method"""
        about = self.testlink_manager.api_about()
        self.assertIsInstance(about, str)
        self.assertEquals(about, DATA['about'])

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_020_method_say_hello(self):
        """TODO: doc method"""
        say_hello = self.testlink_manager.api_say_hello()
        self.assertIsInstance(say_hello, str)
        self.assertEquals(say_hello, DATA['say_hello'])

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_021_method_ping(self):
        """TODO: doc method"""
        ping = self.testlink_manager.api_ping()
        self.assertIsInstance(ping, str)
        self.assertEquals(ping, DATA['ping'])

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_022_method_repeat(self):
        """TODO: doc method"""
        repeat = self.testlink_manager.api_repeat(DATA['repeat'])
        self.assertIsInstance(repeat, str)
        self.assertEquals(
            repeat, "You said: {}".format(DATA['repeat']))


class TestMethodsRaises(TestCase):
    """Test suite for tests methods

    Arguments:
        TestCase {unittest.TestCase} -- base python class for testing
    """

    @classmethod
    def setUpClass(cls):
        """TODO: doc method"""
        cls.testlink_manager = TLManager()

    def setUp(self):
        """TODO: doc method"""
        self.assertIsInstance(
            self.testlink_manager, TLManager)
        self.assertIsInstance(
            self.testlink_manager.log, logging.Logger)

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_001_raises_tproject_notname(self):
        """TODO: doc method"""
        self.assertRaises(
            Exception, self.testlink_manager.api_tproject)

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_002_raises_tproject_emptyname(self):
        """TODO: doc method"""
        self.assertRaises(
            Exception,
            self.testlink_manager.api_tproject,
            '')

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_003_raises_tproject_tplans_notid(self):
        """TODO: doc method"""
        self.assertRaises(
            Exception, self.testlink_manager.api_tproject_tplans)

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_004_raises_tproject_tplans_notfoundid(self):
        """TODO: doc method"""
        self.assertRaises(
            Exception,
            self.testlink_manager.api_tproject_tplans,
            -1)

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_005_raises_tproject_tsuites_first_level_notid(self):
        """TODO: doc method"""
        self.assertRaises(
            Exception, self.testlink_manager.api_tproject_tsuites_first_level)

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_006_raises_tproject_tsuites_first_level_notfoundid(self):
        """TODO: doc method"""
        self.assertRaises(
            Exception,
            self.testlink_manager.api_tproject_tsuites_first_level,
            -1)

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_007_raises_tplan_notname(self):
        """TODO: doc method"""
        self.assertRaises(
            Exception, self.testlink_manager.api_tplan)

    @skipIf(True, 'Test SKIPPED, waiting for issue https://github.com/viglesiasce/testlink/issues/7') # noqa
    def test_008_raises_tplan_emptytprojectname(self):
        """TODO: doc method"""
        self.assertRaises(
            Exception,
            self.testlink_manager.api_tplan,
            '',
            DATA['tplan_name'])

    @skipIf(True, 'Test SKIPPED, waiting for issue https://github.com/viglesiasce/testlink/issues/7') # noqa
    def test_009_raises_tplan_emptytplanname(self):
        """TODO: doc method"""
        self.assertRaises(
            Exception,
            self.testlink_manager.api_tplan,
            DATA['tproject_name'],
            '')

    @skipIf(True, 'Test SKIPPED, waiting for issue https://github.com/viglesiasce/testlink/issues/7') # noqa
    def test_010_raises_tplan_emptytnames(self):
        """TODO: doc method"""
        self.assertRaises(
            Exception,
            self.testlink_manager.api_tplan,
            '', '')

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_011_raises_tplan_platforms_notname(self):
        """TODO: doc method"""
        self.assertRaises(
            Exception, self.testlink_manager.api_tplan_platforms)

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_012_raises_tplan_platforms_notfoundid(self):
        """TODO: doc method"""
        self.assertRaises(
            Exception,
            self.testlink_manager.api_tplan_platforms,
            -1)

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_013_raises_tplan_builds_notid(self):
        """TODO: doc method"""
        self.assertRaises(
            Exception, self.testlink_manager.api_tplan_builds)

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_014_raises_tplan_builds_notfoundid(self):
        """TODO: doc method"""
        self.assertRaises(
            Exception,
            self.testlink_manager.api_tplan_builds,
            -1)

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_015_raises_tplan_tsuites_notid(self):
        """TODO: doc method"""
        self.assertRaises(
            Exception, self.testlink_manager.api_tplan_tsuites)

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_016_raises_tplan_tsuites_notfoundid(self):
        """TODO: doc method"""
        self.assertRaises(
            ResponseException,
            self.testlink_manager.api_tplan_tsuites,
            -1)



# -*- coding: utf-8 -*-
"""Utils tasks for files operations and settings operations"""


import json
from os import path
from sys import version_info


def path_format(file_path=None, file_name=None, is_abspath=False,
                ignore_raises=False):
    """
    Get path joined checking before if path and filepath exist,
     if not, raise an Exception
     if ignore_raise it's enabled, then file_path must include '/' at end lane
    """
    path_formatted = "{}{}".format(file_path, file_name)
    if ignore_raises:
        return path_formatted
    if file_path is None or not path.exists(file_path):
        raise IOError("Path '{}' doesn't exists".format(file_path))
    if file_name is None or not path.exists(path_formatted):
        raise IOError(
            "File '{}{}' doesn't exists".format(file_path, file_name))
    if is_abspath:
        return path.abspath(path.join(file_path, file_name))
    else:
        return path.join(file_path, file_name)


def read_file(is_json=False, file_path=None, encoding='utf-8',
              is_encoding=True):
    """Returns file object from file_path,
       compatible with all py versiones
    optionals:
      can be use to return dict from json path
      can modify encoding used to obtain file
    """
    text = None
    if file_path is None:
        raise Exception("File path received it's None")
    if version_info.major >= 3:
        if not is_encoding:
            encoding = None
        with open(file_path, encoding=encoding) as buff:
            text = buff.read()
    if version_info.major <= 2:
        with open(file_path) as buff:
            if is_encoding:
                text = buff.read().decode(encoding)
            else:
                text = buff.read()
    if is_json:
        return json.loads(text)
    return text


def settings(file_path='qatestlink/configs/', file_name='settings.json',
             is_abspath=True):
    """Returns file settings as a dict to be use on qacode lib"""
    return read_file(is_json=True,
                     file_path=path_format(file_path=file_path,
                                           file_name=file_name,
                                           is_abspath=is_abspath))



# -*- coding: utf-8 -*-
# pylint: disable=invalid-name
"""XMLRPC managers"""


from dicttoxml import dicttoxml
from qatestlink.core.exceptions.response_exception import ResponseException
from qatestlink.core.xmls.route_type import RouteType
import xmltodict


class XMLRPCManager(object):
    """
    Manage all XMLRPCManager requests,
     responses and handle errors. This class
     store all official methods names used
     on original XMLRPC php class
    """

    log = None
    headers = None
    handler = None
    req_dict = None

    def __init__(self, log):
        """Instance for XMLRPC requests manager

        Arguments:
            log {logging.Logger} -- logger func ready to log messages
        """
        self.log = log
        self.headers = {'Content-Type': 'application/xml'}
        self.req_dict = {
            "methodName": "",
            "params": {}
        }

    def parse_response(self, response):
        """Parse response from call to requests library from XML string format
            to 'dict' ready to create/update/find/add/delete Elements on it

        Arguments:
            response {requests.post()} -- Response of call on requests library

        Raises:
            Exception -- if code of response XMLRPC request is not 200

        Returns:
            str -- xml as string text
        """
        if response.status_code != 200:
            raise Exception(
                "status_code invalid: code={}".format(
                    response.status_code))
        return xmltodict.parse(response.text)

    def parse_errors(self, response_as_dict):
        """Find Testlink XMLRPC error structure and raise it's founds

        Arguments:
            response_as_dict {dict} -- dict parsed from XML string

        Raises:
            Exception -- Bad params
            ResponseException -- if error structure it's found
        """
        if not isinstance(response_as_dict, dict):
            raise Exception("Bad param 'response_as_dict' value provided")
        res_value = response_as_dict.get(
            'methodResponse')['params']['param']['value']
        err_info = res_value.get(
            'array')['data']['value']['struct']['member']
        raise ResponseException(
            self.log,
            code=err_info[0]['value']['int'],
            message=err_info[1]['value']['string']
        )

    def req_create(self, dev_key, route_type, params=True):
        """Update property 'req_dict' to add route_type and param or params if
            enabled

        Arguments:
            dev_key {str} -- developer key for Testlink
            route_type {RouteType} -- string to refer XMLRPC testlink call

        Keyword Arguments:
            params {bool} -- enable multiple params using <struct> tree
                (default: {True})
        """
        self.req_dict.update({
            "methodName": route_type.value
        })
        if params:
            self.req_dict.update({
                "params": {
                    "struct": {
                        "member": [
                            {"name": "devKey", "value": dev_key}
                        ]
                    }
                }
            })
            return
        self.req_dict.update({
            "params": {
                "struct": {
                    "name": "devKey",
                    "value": dev_key
                }
            }
        })

    def req_params(self):
        """Obtains params from XML request"""
        params = self.req_dict.get('params')['struct']['member']
        if not params:
            raise ResponseException(
                self.log,
                code=601,
                message="Call to req_create first")
        return params

    def req_check_dev_key(self, dev_key):
        """String xml object ready to use on API call

        Arguments:
            dev_key {str} -- string for Testlink API_KEY

        Returns:
            str -- XML request with parsed params
        """
        self.req_create(dev_key, RouteType.TLINK_CHECK_DEV_KEY, params=False)
        xml = dicttoxml(
            self.req_dict, custom_root='methodCall', attr_type=False)
        return xml

    def req_tprojects(self, dev_key):
        """Obtains all test projects created on remote testlink database,

            TODO: can filter with any property+value combination

        Arguments:
            dev_key {str} -- string of developer key provided by Testlink
                (default: {value obtained from JSON settings file})

        Returns:
            str -- string xml object ready to use on API call
        """
        self.req_dict.update({
            "methodName": RouteType.TPROJECTS.value
        })
        self.req_dict.update({
            "params": {
                "struct": {
                    "name": "devKey",
                    "value": dev_key
                }
            }
        })
        xml = dicttoxml(
            self.req_dict, custom_root='methodCall', attr_type=False)
        return xml

    def req_tproject_by_name(self, dev_key, tproject_name):
        """Obtains all test projects created on remote testlink database, can
            filter by name

        Arguments:
            dev_key {str} -- string of developer key provided by Testlink
                (default: {value obtained from JSON settings file})
            tproject_name {[type]} -- [description]

        Raises:
            Exception -- [description]

        Returns:
            str -- string xml object ready to use on API call
        """
        if not tproject_name:
            raise Exception("Can't call XMLRPC without param, tproject_name")
        self.req_create(dev_key, RouteType.TPROJECT_BY_NAME, params=True)
        self.req_params().append(
            {"name": "testprojectname", "value": tproject_name}
        )
        xml = dicttoxml(
            self.req_dict, custom_root='methodCall', attr_type=False)
        return xml

    def req_tproject_tplans(self, dev_key, tproject_id):
        """Obtains all test plans asigned to test project created on remote
            testlink database, can filter by project id

        Arguments:
            dev_key {str} -- string of developer key provided by Testlink
                (default: {value obtained from JSON settings file})
            tproject_id {str} -- ID of Testlink Test Project to filter Testlink
                Test Plan

        Raises:
            Exception -- [description]

        Returns:
            str -- string xml object ready to use on API call
        """
        if not tproject_id:
            raise Exception("Can't call XMLRPC without param, tproject_id")
        self.req_create(dev_key, RouteType.TPROJECT_TEST_PLANS, params=True)
        self.req_params().append(
            {"name": "testprojectid", "value": tproject_id}
        )
        xml = dicttoxml(
            self.req_dict, custom_root='methodCall', attr_type=False)
        return xml

    def req_tproject_tsuites_first_level(self, dev_key, tproject_id):
        """Obtains all test suites of first level into test project created on
            remote testlink database, can filter by project id

        Arguments:
            Arguments:
            dev_key {str} -- string of developer key provided by Testlink
                (default: {value obtained from JSON settings file})
            tproject_id {str} -- ID of Testlink Test Project to filter Testlink
                Test Plan

        Raises:
            Exception -- [description]

        Returns:
            str -- string xml object ready to use on API call
        """
        if not tproject_id:
            raise Exception("Can't call XMLRPC without param, tproject_id")
        self.req_create(
            dev_key,
            RouteType.TPROJECT_TSUITES_FIRST_LEVEL,
            params=True)
        self.req_params().append(
            {"name": "testprojectid", "value": tproject_id}
        )
        xml = dicttoxml(
            self.req_dict, custom_root='methodCall', attr_type=False)
        return xml

    def req_tplan_by_name(self, dev_key, tproject_name, tplan_name):
        """Obtains all test projects created on remote testlink database, can
            filter by name

        Arguments:
            dev_key {str} -- string of developer key provided by Testlink
                (default: {value obtained from JSON settings file})
            tproject_name {str} -- NAME of Testlink Test Project data
            tplan_name {str} -- NAME of Testlink Test Plan data

        Raises:
            Exception -- [description]

        Returns:
            str -- string xml object ready to use on API call
        """
        if not tproject_name:
            raise Exception("Can't call XMLRPC without param, tproject_name")
        if not tplan_name:
            raise Exception("Can't call XMLRPC without param, tplan_name")
        self.req_create(dev_key, RouteType.TPLAN_BY_NAME, params=True)
        self.req_params().append(
            [
                {"name": "testprojectname", "value": tproject_name},
                {"name": "testplanname", "value": tplan_name}
            ]
        )
        xml = dicttoxml(
            self.req_dict, custom_root='methodCall', attr_type=False)
        return xml

    def req_tplan_platforms(self, dev_key, tplan_id):
        """Obtains all platforms asigned to test plan created on remote
            testlink database, can filter by test plan name

        Arguments:
            dev_key {str} -- string of developer key provided by Testlink
                (default: {value obtained from JSON settings file})
            tplan_id {int} -- ID of Testlink Test Plan data

        Raises:
            Exception -- [description]

        Returns:
            str -- string xml object ready to use on API call
        """
        if not tplan_id:
            raise Exception("Can't call XMLRPC without param, tplan_id")
        self.req_create(dev_key, RouteType.TPLAN_PLATFORMS, params=True)
        self.req_params().append(
            {"name": "testplanid", "value": tplan_id}
        )
        xml = dicttoxml(
            self.req_dict, custom_root='methodCall', attr_type=False)
        return xml

    def req_tplan_builds(self, dev_key, tplan_id):
        """Obtains all platforms asigned to test plan created on remote
            testlink database, can filter by test plan id

        Arguments:
            dev_key {str} -- string of developer key provided by Testlink
                (default: {value obtained from JSON settings file})
            tplan_id {int} -- ID of Testlink Test Plan data

        Raises:
            Exception -- [description]

        Returns:
            str -- string xml object ready to use on API call
        """
        if not tplan_id:
            raise Exception("Can't call XMLRPC without param, tplan_id")
        self.req_create(dev_key, RouteType.TPLAN_BUILDS, params=True)
        self.req_params().append(
            {"name": "testplanid", "value": tplan_id}
        )
        xml = dicttoxml(
            self.req_dict, custom_root='methodCall', attr_type=False)
        return xml

    def req_tplan_tsuites(self, dev_key, tplan_id):
        """Obtains all test suites asigned to test plan created on remote
            testlink database, can filter by test plan id

        Arguments:
            dev_key {str} -- string of developer key provided by Testlink
                (default: {value obtained from JSON settings file})
            tplan_id {int} -- ID of Testlink Test Plan data

        Raises:
            Exception -- [description]

        Returns:
            str -- string xml object ready to use on API call
        """
        if not tplan_id:
            raise Exception("Can't call XMLRPC without param, tplan_id")
        self.req_create(dev_key, RouteType.TPLAN_BUILDS, params=True)
        self.req_params().append(
            {"name": "testplanid", "value": tplan_id}
        )
        xml = dicttoxml(
            self.req_dict, custom_root='methodCall', attr_type=False)
        return xml

    def req_tplan_tcases(self, dev_key, tplan_id):
        """Obtains all test cases asigned to test plan created on remote
            testlink database, can filter by test plan id

        Arguments:
            dev_key {str} -- string of developer key provided by Testlink
                (default: {value obtained from JSON settings file})
            tplan_id {int} -- ID of Testlink Test Plan data

        Raises:
            Exception -- [description]

        Returns:
            str -- string xml object ready to use on API call
        """
        if not tplan_id:
            raise Exception("Can't call XMLRPC without param, tplan_id")
        self.req_create(dev_key, RouteType.TPLAN_TCASES, params=True)
        self.req_params().append(
            {"name": "testplanid", "value": tplan_id}
        )
        xml = dicttoxml(
            self.req_dict, custom_root='methodCall', attr_type=False)
        return xml

    def req_tplan_build_latest(self, dev_key, tplan_id):
        """Obtains latest build by choosing the maximum build id for a specific
            test plan remote testlink database, can filter by test plan id

        Arguments:
            dev_key {str} -- string of developer key provided by Testlink
                (default: {value obtained from JSON settings file})
            tplan_id {int} -- ID of Testlink Test Plan data

        Raises:
            Exception -- Bad params

        Returns:
            str -- string xml object ready to use on API call
        """
        if not tplan_id:
            raise Exception("Can't call XMLRPC without param, tplan_id")
        self.req_create(dev_key, RouteType.TPLAN_BUILD_LATEST, params=True)
        self.req_params().append(
            {"name": "testplanid", "value": tplan_id}
        )
        xml = dicttoxml(
            self.req_dict, custom_root='methodCall', attr_type=False)
        return xml

    def req_tplan_totals(self, dev_key, tplan_id):
        """Obtains latest totals by choosing the maximum tplan id on remote
            testlink database, can filter by test plan id

        Arguments:
            dev_key {str} -- string of developer key provided by Testlink
                (default: {value obtained from JSON settings file})
            tplan_id {int} -- ID of Testlink Test Plan data

        Raises:
            Exception -- Bad params

        Returns:
            str -- string xml object ready to use on API call
        """
        if not tplan_id:
            raise Exception("Can't call XMLRPC without param, tplan_id")
        self.req_create(dev_key, RouteType.TPLAN_TOTALS, params=True)
        self.req_params().append(
            {"name": "testplanid", "value": tplan_id}
        )
        xml = dicttoxml(
            self.req_dict, custom_root='methodCall', attr_type=False)
        return xml

    def req_tsuite_by_id(self, dev_key, tsuite_id):
        """Obtains one test suite created on remote testlink database, can
            filter by test plan id

        Arguments:
            dev_key {str} -- string of developer key provided by Testlink
                (default: {value obtained from JSON settings file})
            tsuite_id {int} -- ID of Testlink Test Suite data

        Raises:
            Exception -- Bad params

        Returns:
            str -- string xml object ready to use on API call
        """
        if not tsuite_id:
            raise Exception("Can't call XMLRPC without param, tsuite_id")
        self.req_create(dev_key, RouteType.TSUITE_BY_ID, params=True)
        self.req_params().append(
            {"name": "testsuiteid", "value": tsuite_id}
        )
        xml = dicttoxml(
            self.req_dict, custom_root='methodCall', attr_type=False)
        return xml

    def req_tsuite_tsuites_by_id(self, dev_key, tsuite_id):
        """Obtains all test suites down of one test suite created on remote
            testlink database, can filter by test plan id

        Arguments:
            dev_key {str} -- string of developer key provided by Testlink
                (default: {value obtained from JSON settings file})
            tsuite_id {int} -- ID of Testlink Test Suite data

        Raises:
            Exception -- Bad params

        Returns:
            str -- string xml object ready to use on API call
        """
        if not tsuite_id:
            raise Exception("Can't call XMLRPC without param, tsuite_id")
        self.req_create(dev_key, RouteType.TSUITE_TSUITES, params=True)
        self.req_params().append(
            {"name": "testsuiteid", "value": tsuite_id}
        )
        xml = dicttoxml(
            self.req_dict, custom_root='methodCall', attr_type=False)
        return xml

    def req_tcase_by_id_or_external(self, dev_key,
                                    tcase_id=None, external_id=None):
        """Obtains one test case created on remote testlink database, can
            filter by test case id (int) or external id (str)

        Arguments:
            dev_key {str} -- string of developer key provided by Testlink
                (default: {value obtained from JSON settings file})
            tcase_id {int} -- ID of Testlink Test Case data (default: None)
            external_id {str} -- tc_full_external_id of Testlink Test Case
                data (default: None)

        Raises:
            Exception -- Bad params

        Returns:
            str -- string xml object ready to use on API call
        """
        if tcase_id and external_id:
            raise Exception(
                ("Can't call XMLRPC without both params,"
                 "choose one of : [tcase_id, external_id]"))
        if not tcase_id and not external_id:
            raise Exception(
                ("Can't call XMLRPC without any params,"
                 "choose one of : [tcase_id, external_id]"))
        self.req_create(dev_key, RouteType.TCASE_BY_IDS, params=True)
        if isinstance(tcase_id, int):
            self.req_params().append(
                {"name": "testcaseid", "value": int(tcase_id)}
            )
        if external_id:
            self.req_params().append(
                {"name": "testcaseexternalid", "value": str(external_id)}
            )
        xml = dicttoxml(
            self.req_dict, custom_root='methodCall', attr_type=False)
        return xml

    def req_tcase_by_name(self, dev_key, tcase_name):
        """Obtains one test case created on remote testlink database, can
            filter by test case name (str)

        Arguments:
            dev_key {str} -- string of developer key provided by Testlink
                (default: {value obtained from JSON settings file})
            tcase_name {str} -- NAME of Testlink Test Case data

        Raises:
            Exception -- Bad params

        Returns:
            str -- string xml object ready to use on API call
        """
        if not tcase_name:
            raise Exception("Can't call XMLRPC without param, tcase_name")
        self.req_create(dev_key, RouteType.TCASE_ID_BY_NAME, params=True)
        self.req_params().append(
            {"name": "testcasename", "value": tcase_name}
        )
        xml = dicttoxml(
            self.req_dict, custom_root='methodCall', attr_type=False)
        return xml

    def req_tcase_report(self, **kwargs):
        """Reports a result for a single test case

        Keyword Arguments:
            tcase_id {[type]} -- [description] (default: {None})
            external_id {[type]} -- [description] (default: {None})
            tplan_id {[type]} -- [description] (default: {None})
            status {[type]} -- [description] (default: {None})
            build_id {[type]} -- [description] (default: {None})
            build_name {[type]} -- [description] (default: {None})
            notes {[type]} -- [description] (default: {None})
            duration {[type]} -- [description] (default: {None})
            guess {[type]} -- [description] (default: {None})
            bug_id {[type]} -- [description] (default: {None})
            platform_id {[type]} -- [description] (default: {None})
            platform_name {[type]} -- [description] (default: {None})
            custom_fields {[type]} -- [description] (default: {None})
            overwrite {[type]} -- [description] (default: {None})
            user_name {[type]} -- [description] (default: {None})
            timestamp {[type]} -- [description] (default: {None})
            dev_key {[type]} -- [description] (default: {None})

        Returns:
            [type] -- [description]
        """
        if kwargs.get("tcase_id") and kwargs.get("external_id"):
            raise Exception(
                ("Can't call XMLRPC without both params,"
                 "choose one of : [tcase_id, external_id]"))
        if not kwargs.get("tcase_id") and not kwargs.get("external_id"):
            raise Exception(
                ("Can't call XMLRPC without any params,"
                 "choose one of : [tcase_id, external_id]"))
        if not kwargs.get("tplan_id"):
            raise Exception(
                "Can't call XMLRPC without any param 'tplan_id' ]")
        if not kwargs.get("status"):
            raise Exception(
                "Can't call XMLRPC without any param 'status' ]")
        if not kwargs.get("build_id"):
            raise Exception(
                "Can't call XMLRPC without any param 'build_id' ]")
        if not kwargs.get("platform_id"):
            raise Exception(
                "Can't call XMLRPC without any param 'platform_id' ]")
        self.req_create(
            kwargs.get("dev_key"),
            RouteType.TCASE_REPORT_RESULT,
            params=True)
        if kwargs.get("tcase_id"):
            self.req_params().append(
                {"name": "testcaseid", "value": int(kwargs.get("tcase_id"))}
            )
        if kwargs.get("external_id"):
            self.req_params().append(
                {"name": "testcaseexternalid", "value": str(
                    kwargs.get("external_id"))}
            )
        if kwargs.get("tplan_id"):
            self.req_params().append(
                {"name": "testplanid", "value": str(kwargs.get("tplan_id"))}
            )
        if kwargs.get("status"):
            self.req_params().append(
                {"name": "status", "value": str(kwargs.get("status"))}
            )
        if kwargs.get("build_id"):
            self.req_params().append(
                {"name": "buildid", "value": int(kwargs.get("build_id"))}
            )
        if kwargs.get("build_name"):
            self.req_params().append(
                {"name": "buildname", "value": str(kwargs.get("build_name"))}
            )
        if kwargs.get("notes"):
            self.req_params().append(
                {"name": "notes", "value": str(kwargs.get("notes"))}
            )
        if kwargs.get("duration"):
            self.req_params().append(
                {"name": "execduration", "value": int(kwargs.get("duration"))}
            )
        if kwargs.get("guess"):
            self.req_params().append(
                {"name": "guess", "value": int(kwargs.get("guess"))}
            )
        if kwargs.get("bug_id"):
            self.req_params().append(
                {"name": "bugid", "value": int(kwargs.get("bug_id"))}
            )
        if kwargs.get("platform_id"):
            self.req_params().append(
                {"name": "platformid", "value": int(
                    kwargs.get("platform_id"))}
            )
        if kwargs.get("platform_name"):
            self.req_params().append(
                {"name": "platformname", "value": str(
                    kwargs.get("platform_name"))}
            )
        if kwargs.get("custom_fields"):
            # noqa : param real name 'customfields'
            # noqa : array of member>(name+value>type)
            raise NotImplementedError("Open an issue at Github")
        if kwargs.get("overwrite"):
            self.req_params().append(
                {"name": "overwrite", "value": bool(
                    kwargs.get("overwrite"))}
            )
        if kwargs.get("user_name"):
            self.req_params().append(
                {"name": "user", "value": str(kwargs.get("user_name"))}
            )
        if kwargs.get("timestamp"):
            self.req_params().append(
                {"name": "timestamp", "value": str(kwargs.get("timestamp"))}
            )
        xml = dicttoxml(
            self.req_dict, custom_root='methodCall', attr_type=False)
        return xml

    def req_user_exist(self, dev_key, user_name):
        """String xml object ready to use on API call

        Arguments:
            dev_key {str} -- string for Testlink API_KEY
            user_name {str} -- NAME of Testlink user

        Returns:
            str -- XML request with parsed params
        """
        self.req_create(dev_key, RouteType.TLINK_USER_EXIST, params=True)
        self.req_params().append(
            {"name": "user", "value": user_name}
        )
        xml = dicttoxml(
            self.req_dict, custom_root='methodCall', attr_type=False)
        return xml

    def req_about(self):
        """String xml object ready to use on API call

        Returns:
            str -- XML request with parsed params
        """
        self.req_create(None, RouteType.TLINK_ABOUT, params=False)
        xml = dicttoxml(
            self.req_dict, custom_root='methodCall', attr_type=False)
        return xml

    def req_say_hello(self):
        """String xml object ready to use on API call

        Returns:
            str -- XML request with parsed params
        """
        self.req_create(None, RouteType.TLINK_SAY_HELLO, params=False)
        xml = dicttoxml(
            self.req_dict, custom_root='methodCall', attr_type=False)
        return xml

    def req_ping(self):
        """String xml object ready to use on API call

        Returns:
            str -- XML request with parsed params
        """
        self.req_create(None, RouteType.TLINK_PING, params=False)
        xml = dicttoxml(
            self.req_dict, custom_root='methodCall', attr_type=False)
        return xml

    def req_repeat(self, repeat):
        """String xml object ready to use on API call

        Returns:
            str -- XML request with parsed params
        """
        if not repeat:
            raise Exception("Can't call XMLRPC without param, repeat")
        self.req_create(None, RouteType.TLINK_REPEAT, params=True)
        self.req_params().append(
            {"name": "str", "value": repeat}
        )
        xml = dicttoxml(
            self.req_dict, custom_root='methodCall', attr_type=False)
        return xml



# -*- coding: utf-8 -*-
"""All method names used on official XMLRPC calls"""


from enum import Enum


class RouteType(Enum):
    """
    Official doc:
        https://github.com/viglesiasce/testlink/blob/master/lib/api/xmlrpc.class.php

    All REQUEST methods names to use on classes:
        XmlBase
            XmlRequest
            XmlResponse

    Documented Enumerator
        use intellisense to improve development
    """

    # TestProjects

    """TODO
    Args:
        devKey:string
            authorization dev key
    """
    TPROJECT_CREATE = 'tl.createTestProject'

    """TODO
    Args:
        devKey:string
            authorization dev key
    """
    TPROJECTS = 'tl.getProjects'

    """TODO
    Args:
        devKey:string
            authorization dev key
    """
    TPROJECT_BY_NAME = 'tl.getTestProjectByName'

    """TODO
    Args:
        devKey:string
            authorization dev key
    """
    TPROJECT_TEST_PLANS = 'tl.getProjectTestPlans'

    """TODO
    Args:
        devKey:string
            authorization dev key
    """
    TPROJECT_TSUITES_FIRST_LEVEL = 'tl.getFirstLevelTestSuitesForTestProject'

    # TestPlans

    """TODO
    Args:
        devKey:string
            authorization dev key
    """
    TPLAN_CREATE = 'tl.createTestPlan'

    """TODO
    Args:
        devKey:string
            authorization dev key
    """
    TPLAN_ADD_TCASE = 'tl.addTestCaseToTestPlan'

    """TODO
    Args:
        devKey:string
            authorization dev key
    """
    TPLAN_BY_NAME = 'tl.getTestPlanByName'

    """TODO
    Args:
        devKey:string
            authorization dev key
    """
    TPLAN_PLATFORMS = 'tl.getTestPlanPlatforms'

    """TODO
    Args:
        devKey:string
            authorization dev key
    """
    TPLAN_TOTALS = 'tl.getTotalsForTestPlan'

    """TODO
    Args:
        devKey:string
            authorization dev key
    """
    TPLAN_BUILDS = 'tl.getBuildsForTestPlan'

    """TODO
    Args:
        devKey:string
            authorization dev key
    """
    TPLAN_BUILD_LATEST = 'tl.getLatestBuildForTestPlan'

    """TODO
    Args:
        devKey:string
            authorization dev key
    """
    TPLAN_EXEC_LATEST = 'tl.getLastExecutionResult'

    """TODO
    Args:
        devKey:string
            authorization dev key
    """
    TPLAN_TSUITES = 'tl.getTestSuitesForTestPlan'

    """TODO
    Args:
        devKey:string
            authorization dev key
    """
    TPLAN_TCASES = 'tl.getTestCasesForTestPlan'
    # Builds

    """TODO
    Args:
        devKey:string
            authorization dev key
    """
    BUILD_CREATE = 'tl.createBuild'
    # Testsuites

    """TODO
    Args:
        devKey:string
            authorization dev key
    """
    TSUITE_CREATE = 'tl.createTestSuite'

    """TODO
    Args:
        devKey:string
            authorization dev key
    """
    TSUITE_TSUITES = 'tl.getTestSuitesForTestSuite'

    """TODO
    Args:
        devKey:string
            authorization dev key
    """
    TSUITE_BY_ID = 'tl.getTestSuiteByID'

    """TODO
    Args:
        devKey:string
            authorization dev key
    """
    TSUITE_TCASES = 'tl.getTestCasesForTestSuite'
    # Requirements

    """TODO
    Args:
        devKey:string
            authorization dev key
    """
    REQ_ASSIGN = 'tl.assignRequirements'
    # Testcases

    """TODO
    Args:
        devKey:string
            authorization dev key
    """
    TCASE_REPORT_RESULT = 'tl.reportTCResult'

    """TODO
    Args:
        devKey:string
            authorization dev key
    """
    TCASE_EXEC_RESULT = 'tl.setTestCaseExecutionResult'

    """TODO
    Args:
        devKey:string
            authorization dev key
    """
    TCASE_CREATE = 'tl.createTestCase'

    """TODO
    Args:
        devKey:string
            authorization dev key
    """
    TCASE_ID_BY_NAME = 'tl.getTestCaseIDByName'

    """TODO
    Args:
        devKey:string
            authorization dev key
    """
    TCASE_CUSTOM_FIELD_DESIGN_VALUE = 'tl.getTestCaseCustomFieldDesignValue'

    """TODO
    Args:
        devKey:string
            authorization dev key
    """
    TCASE_ATTACHMENTS = 'tl.getTestCaseAttachments'

    """
    Args:
        devKey:string
            authorization dev key
        testcaseid:int
            optional, if does not is present
            testcaseexternalid must be present
        testcaseexternalid:int
            optional, if does not is present
            testcaseid must be present
        version:int
            optional, if does not is present
            max version number will be retuned
    """
    TCASE_BY_IDS = 'tl.getTestCase'

    # Utils
    """TODO:
    I really don't know objetive of this method
    Also don't know about mixed type
    Args:
        devKey:string
            authorization dev key
        nodeID:mixed
    """
    TLINK_FULL_PATH = 'tl.getFullPath'

    """
    Args:
        devKey:string
            authorization dev key
        executionid: int
    Response:
       resultInfo:mixed
           status: true/false of success
           id: result id or error code
           message: optional message for error message string
    """
    TLINK_EXEC_DELETE = 'tl.deleteExecution'

    """
    Args:
        devKey:string
            authorization dev key
        user:string
            user name
    Response:
        true if everything OK, otherwise error structure
    """
    TLINK_USER_EXIST = 'tl.doesUserExist'

    """
    Args:
        devKey:string
            authorization dev key
    Response:
        true if everything OK, otherwise error structure
    """
    TLINK_CHECK_DEV_KEY = 'tl.checkDevKey'

    """
    Args:
        without args
    Response:
        string
             Testlink API Version {version} initially written
             by Asiel Brumfield with contributions by
             TestLink development Team
    """
    TLINK_ABOUT = 'tl.about'

    """
    TODO: official developer said this don't work
    This method is meant primarily for testing
    and debugging during development
    Args:
        without args
    Response:
        bool: if state changed returns True, False if not
    """
    TLINK_TMODE_TOGGLE = 'tl.setTestMode'

    """
    Just and alias for TLINK_SAY_HELLO request
    Args:
        without args
    Response:
        string: "Hello!"
    """
    TLINK_PING = 'tl.ping'

    """
    Just return a message
    Args:
        without args
    Response:
        string: "Hello!"
    """
    TLINK_SAY_HELLO = 'tl.sayHello'

    """
    Just return your message contained on another message
    Args:
        str: string message to be contained
    Response:
        string: "You said: {str arg}"
    """
    TLINK_REPEAT = 'tl.repeat'



# -*- coding: utf-8 -*-
"""module qatestlink.core.connections"""


import requests


class ConnectionBase(object):
    """TODO: doc class"""

    _host = None
    _port = None
    _xmlrpc_route = None
    _is_https = None
    _url = None
    _log = None

    def __init__(self, log, host='localhost', port=80,
                 xmlrpc_route='lib/api/xmlrpc/v1/xmlrpc.php', is_https=False):
        """TODO: doc method"""
        self._host = host
        self._port = port
        self._xmlrpc_route = xmlrpc_route
        self._is_https = is_https
        self._log = log
        # Format Testlink XMLRPC url
        self._url = self.__url_format()

    def __url_format(self):
        url = "{}{}:{}/{}"
        url_prefix = 'http://'
        if self._is_https:
            url_prefix = 'https://'
        return url.format(
            url_prefix,
            self._host,
            self._port,
            self._xmlrpc_route)

    def post(self, headers, req_data):
        """
        :Args:
            headers: {'Content-Type':'application/xml'}
            data: <methodCall>...</methodCall>
        """
        self._log.info("POST request:")
        self._log.debug("    url={}".format(self._url))
        self._log.debug("    headers={}".format(headers))
        self._log.debug("    data={}".format(req_data))
        response = requests.post(
            self._url,
            headers=headers,
            data=req_data)
        self._log.info("POST response:")
        self._log.info("    status_code={}".format(
            response.status_code))
        self._log.debug("    data={}".format(
            response.text))
        return response



# -*- coding: utf-8 -*-
"""module qatestlink.core.utils.logger_manager"""


import logging


class LoggerManager(object):
    """
    Start logger named 'qatestlink'
     with DEBUG level and just with console reporting
    """

    log = None

    def __init__(self, log_level=None):
        """Start logger"""
        formatter = logging.Formatter(
            '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        logger = logging.getLogger('qatestlink')
        logger_stream = logging.StreamHandler()
        if log_level is None or log_level == 'DEBUG':
            log_level = logging.DEBUG
        elif log_level == 'INFO':
            log_level = logging.INFO
        elif log_level == 'WARNING':
            log_level = logging.WARNING
        elif log_level == 'ERROR':
            log_level = logging.ERROR
        elif log_level == 'CRITICAL':
            log_level = logging.CRITICAL
        logger.setLevel(log_level)
        logger_stream.setLevel(log_level)
        logger_stream.setFormatter(formatter)
        for old_handler in logger.handlers:
            logger.removeHandler(old_handler)
        logger.addHandler(logger_stream)
        # alias to improve logging calls
        self.log = logger



# -*- coding: utf-8 -*-
"""TODO: doc module"""


import logging
from unittest import TestCase
from unittest import skipIf
from qatestlink.core.testlink_manager import TLManager
from qatestlink.core.utils import settings


SETTINGS = settings()
API_DEV_KEY = SETTINGS['dev_key']
SKIP = SETTINGS['tests']['skip']['connection']
SKIP_MESSAGE = SETTINGS['tests']['skip_message']


class TestConnection(TestCase):
    """TODO: doc class"""

    @classmethod
    def setUpClass(cls):
        """TODO: doc method"""
        cls.testlink_manager = TLManager()

    def setUp(self):
        """TODO: doc method"""
        self.assertIsInstance(
            self.testlink_manager, TLManager)
        self.assertIsInstance(
            self.testlink_manager.log, logging.Logger)

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_001_connok_bysettings(self):
        """TODO: doc method"""
        self.assertTrue(
            self.testlink_manager.api_login())

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_002_connok_byparam(self):
        """TODO: doc method"""
        self.assertTrue(
            self.testlink_manager.api_login(
                dev_key=API_DEV_KEY))

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_003_connok_notdevkey(self):
        """TODO: doc method"""
        self.assertTrue(
            self.testlink_manager.api_login(
                dev_key=None))


class TestConnectionRaises(TestCase):
    """TODO: doc class"""

    @classmethod
    def setUpClass(cls):
        """TODO: doc method"""
        cls.testlink_manager = TLManager()

    def setUp(self):
        """TODO: doc method"""
        self.assertIsInstance(
            self.testlink_manager, TLManager)
        self.assertIsInstance(
            self.testlink_manager.log, logging.Logger)

    @skipIf(SKIP, SKIP_MESSAGE)
    def test_001_raises_connemptydevkey(self):
        """TODO: doc method"""
        self.assertRaises(
            Exception,
            self.testlink_manager.api_login, dev_key=' ')



# -*- coding: utf-8 -*-
"""module qatestlink.core.models.tl_models"""


import re


class ModelBase(object):
    """TODO: doc class"""

    # Testlink common object properties
    id = None
    name = None

    def __init__(self, properties, properties_int=None, properties_bool=None,
                 load=True):
        """TODO: doc method"""
        if properties is None:
            raise Exception('Bad param, res_member can\'t be None')
        if len(properties) <= 0:
            raise Exception(
                'Bad param, res_member can\'t be empty list')
        self._properties = properties
        if not properties_int:
            properties_int = []
        self._properties_int = properties_int
        if not properties_bool:
            properties_bool = []
        self._properties_bool = properties_bool
        if load:
            self._load()

    def _load(self):
        """Load properties setting all as object properties"""
        for res_property in self._properties:
            name = self.convert_name(res_property['name'])
            value = res_property['value']
            if name in self._properties_int:
                if value.get('string'):
                    setattr(self, name, int(value['string']))
                else:
                    setattr(self, name, int(value['int']))
            elif name in self._properties_bool:
                if value.get('string'):
                    setattr(self, name, bool(value['string']))
                else:
                    setattr(self, name, bool(value['boolean']))
            else:
                if value.get('string', None):
                    setattr(self, name, value['string'])
                if value.get('struct', None):
                    setattr(self, name, 'NOT_IMPLEMENTED')

    def convert_name(self, name):
        """Take an string in CamelCase notation to transform to snake_case

        Arguments:
            name {str} -- text at CamelCase notation

        Returns:
            str -- text transformed to snake_case
        """
        first_cap_re = re.compile('(.)([A-Z][a-z]+)')
        all_cap_re = re.compile('([a-z0-9])([A-Z])')
        s1 = first_cap_re.sub(r'\1_\2', name)
        return all_cap_re.sub(r'\1_\2', s1).lower()


class TProject(ModelBase):
    """TODO: doc class"""

    # Testlink object properties
    is_public = None
    notes = None
    color = None
    active = None
    option_reqs = None
    option_priority = None
    option_automation = None
    options = None
    prefix = None
    tc_counter = None
    issue_tracker_enabled = None
    reqmgr_integration_enabled = None
    api_key = None
    opt = None # noqa

    def __init__(self, properties, properties_int=None, properties_bool=None):
        """TODO: doc method"""
        super(TProject, self).__init__(
            properties,
            properties_int=['id', 'tc_counter'],
            properties_bool=[
                'is_public',
                'active',
                'option_reqs',
                'option_priority',
                'option_automation',
                'issue_tracker_enabled',
                'reqmgr_integration_enabled'
            ]
        )

    def __repr__(self):
        """Show basic properties for this object

        Returns:
            str -- format text with values for
                'TProject: id={}, name={}, is_public={}'
        """
        return "TProject: id={}, name={}, is_public={}".format(
            self.id,
            self.name,
            self.is_public)


class TPlan(ModelBase):
    """TODO: doc class"""

    # Testlink object properties
    is_public = None
    active = None
    tproject_id = None
    notes = None

    def __init__(self, properties):
        """TODO: doc method"""
        super(TPlan, self).__init__(
            properties,
            properties_int=['id', 'testproject_id'],
            properties_bool=['is_public', 'active']
        )

    def _load(self):
        super(TPlan, self)._load()
        for res_property in self._properties:
            name = self.convert_name(res_property['name'])
            value = res_property['value']
            if name == 'testproject_id':
                setattr(self, 'tproject_id', int(value['string']))

    def __repr__(self):
        """Show basic properties for this object

        Returns:
            str -- format text with values for
                'TPlan: id={}, name={}, is_public={}'
        """
        return "TPlan: id={}, name={}, is_public={}".format(
            self.id,
            self.name,
            self.is_public)


class TSuite(ModelBase):
    """TODO: doc class"""

    # Testlink object properties
    parent_id = None
    node_type_id = None
    node_order = None
    node_table = None
    is_public = None
    active = None

    def __init__(self, properties):
        """TODO: doc method"""
        super(TSuite, self).__init__(
            properties,
            properties_int=[
                'id',
                'parent_id',
                'node_type_id',
                'node_order'
            ],
            properties_bool=['is_public', 'active']
        )

    def __repr__(self):
        """Show basic properties for this object

        Returns:
            str -- format text with values for
                'TSuite: id={}, name={}, parent_id={}'
        """
        return "TSuite: id={}, name={}, parent_id={}".format(
            self.id,
            self.name,
            self.parent_id)


class TPlatform(ModelBase):
    """TODO: doc class"""

    # Testlink object properties
    notes = None

    def __init__(self, properties):
        """TODO: doc method"""
        super(TPlatform, self).__init__(
            properties,
            properties_int=['id'],
            properties_bool=[]
        )

    def __repr__(self):
        """Show basic properties for this object

        Returns:
            str -- format text with values for
                'TPlatform: id={}, name={}, notes={}'
        """
        return "TPlatform: id={}, name={}".format(
            self.id,
            self.name)


class TBuild(ModelBase):
    """TODO: doc class"""

    # Testlink object properties
    notes = None
    testplan_id = None
    active = None
    is_open = None
    release_date = None
    closed_on_date = None
    creation_ts = None

    def __init__(self, properties):
        """TODO: doc method"""
        super(TBuild, self).__init__(
            properties,
            properties_int=['id', 'testplan_id'],
            properties_bool=['active', 'is_open']
        )

    def __repr__(self):
        """Show basic properties for this object

        Returns:
            str -- format text with values for
                'TBuild: id={}, name={}, testplan_id={}'
        """
        return "TBuild: id={}, name={}, testplan_id={}".format(
            self.id,
            self.name,
            self.testplan_id)


class TCase(ModelBase):
    """TODO: doc class"""

    # Testlink object properties
    notes = None

    def __init__(self, properties):
        """TODO: doc method"""
        super(TCase, self).__init__(
            properties,
            properties_int=['id', 'tcase_id'],
            properties_bool=None
        )

    def _load(self):
        super(TCase, self)._load()
        for res_property in self._properties:
            name = self.convert_name(res_property['name'])
            value = res_property['value']
            if name == 'testcase_id':
                setattr(self, 'id', int(value['string']))
            if name == 'testcase_name':
                setattr(self, 'name', value['string'])
            if name == 'full_tc_external_id':
                setattr(self, 'external_id', value['string'])

    def __repr__(self):
        """Show basic properties for this object

        Returns:
            str -- format text with values for
                'TCase: id={}, name={}, notes={}'
        """
        return "TCase: id={}, name={}, notes={}".format(
            self.id,
            self.name,
            self.notes)



# -*- coding: utf-8 -*-
# pylint: disable=broad-except
"""qatestlink module can be installed and configured from here"""

from os import path
from setuptools import setup, find_packages
from sys import version_info
from qatestlink.core.utils import read_file
from qatestlink.core.utils import path_format


VERSION = '0.0.9'
CURR_PATH = "{}{}".format(path.abspath(path.dirname(__file__)), '/')
INSTALL_REQUIRES = [
    'requests',
    'pytest',
    'xmltodict',
    'dicttoxml',
]

def read(file_name=None, is_encoding=True, ignore_raises=False):
    """Read file"""
    if file_name is None:
        raise Exception("File name not provided")
    if ignore_raises:
        try:
            return read_file(is_encoding=is_encoding,
                             file_path=path_format(
                                 file_path=CURR_PATH,
                                 file_name=file_name,
                                 ignore_raises=ignore_raises))
        except Exception:
            # TODO: not silence like this,
            # must be on setup.cfg, README path
            return 'NOTFOUND'
    return read_file(is_encoding=is_encoding,
                     file_path=path_format(
                         file_path=CURR_PATH,
                         file_name=file_name,
                         ignore_raises=ignore_raises))

def get_install_requires():
    """Get a list of pypi python package dependencies
    Returns:
        list -- list of dependecy package names
    """
    if version_info <= (3, 4):
        INSTALL_REQUIRES.append('enum34')
    return INSTALL_REQUIRES


setup(
    name='qatestlink',
    version=VERSION,
    license=read("LICENSE", is_encoding=False, ignore_raises=True),
    packages=find_packages(exclude=['tests']),
    description='Main automation lib',
    long_description=read("README.rst"),
    author='Netzulo Open Source',
    author_email='netzuleando@gmail.com',
    url='https://github.com/netzulo/qatestlink',
    download_url='https://github.com/netzulo/qatestlink/tarball/v{}'.format(
        VERSION),
    keywords=[
        'testing',
        'logging',
        'functional',
        'http',
        'test',
        'testlink',
        'XMLRPC',
        'requests'
    ],
    install_requires=get_install_requires(),
    setup_requires=[
        'tox',
        'pytest-runner'
    ],
    tests_require=[
        'pytest-coverage',
        'pytest-html',
        'pytest-dependency',
        'flake8'
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
    ],
)



# -*- coding: utf-8 -*-
"""module qatestlink.core.models.tl_reports"""


from .tl_models import ModelBase


class ReportBase(ModelBase):
    """TODO: doc class"""

    totals = None

    def __init__(self, properties, properties_int=None, properties_bool=None,
                 load=True):
        """TODO: doc method"""
        super(ReportBase, self).__init__(
            properties,
            properties_int=properties_int,
            properties_bool=properties_bool,
            load=load)
        self.totals = {}

    def __repr__(self):
        """Show basic properties for this object

        Returns:
            str -- format text with values for
                'ReportBase: totals:dict={}'
        """
        return "ReportBase: totals:dict={}".format(self.totals)


class RTPlanTotals(ReportBase):
    """TODO: doc class"""

    by_tester = None

    def __init__(self, properties, properties_int=None, properties_bool=None,
                 load=False):
        """TODO: doc method"""
        super(RTPlanTotals, self).__init__(
            properties,
            properties_int=properties_int,
            properties_bool=properties_bool,
            load=load)
        self.by_tester = list()
        # just load from this class instead of parent class
        self._load()

    def _load(self):
        super(RTPlanTotals, self)._load()
        for res_property in self._properties:
            name = self.convert_name(res_property['name'])
            value = res_property['value']
            if name == 'with_tester':
                with_tester_members = value['struct']['member']
                by_testers = list()
                for member in with_tester_members:
                    user_id = member['name']
                    report_totals_members = member.get(
                        'value')['struct']['member']
                    by_tester = {
                        "user_id": int(user_id),
                        "report_types": []
                    }
                    for member_total in report_totals_members:
                        report_type = member_total['name']
                        report_data_members = member_total.get(
                            'value')['struct']['member']
                        platform_id = None
                        report_type = None
                        qty = None
                        for report_data_member in report_data_members:
                            prop_name = report_data_member['name']
                            prop_value = report_data_member['value']
                            if prop_name == 'status':
                                report_type = prop_value['string']
                            if prop_name == 'platform_id':
                                try:
                                    platform_id = prop_value['string']
                                except KeyError:
                                    platform_id = prop_value['int']
                            if prop_name == 'exec_qty':
                                try:
                                    qty = prop_value['string']
                                except KeyError:
                                    qty = prop_value['int']
                        by_tester['report_types'].append(
                            {
                                "report_type": str(report_type),
                                "platform_id": int(platform_id),
                                "qty": int(qty)
                            }
                        )
                    by_testers.append(by_tester)
                setattr(self, 'by_tester', by_testers)
                setattr(self, 'with_tester', by_testers)

    def __repr__(self):
        """Show basic properties for this object

        Returns:
            str -- format text with values for
                'ReportBase: totals:dict={}'
        """
        return "RTPlanTotals: by_tester:dict={}".format(self.totals)


class RTCase(ReportBase):
    """TODO: doc class"""

    status = None
    operation = None
    overwrite = None
    message = None

    def __init__(self, properties, properties_int=None, properties_bool=None,
                 load=True):
        """TODO: doc method"""
        super(RTCase, self).__init__(
            properties,
            properties_int=['id'],
            properties_bool=['status', 'overwrite']
        )

    def __repr__(self):
        """Show basic properties for this object

        Returns:
            str -- format text with values for
                'ReportBase: totals:dict={}'
        """
        return ("RTCase: id={}, status:bool={}, operation={},"
                "overwrite={}, message={}").format(
                    self.id,
                    self.status,
                    self.operation,
                    self.overwrite,
                    self.message)
